/*
 * Copyright (C) 2005, Wei Yuan.
 * All rights reserved.
 * 
 * Author: Wei Yuan
 * Created on: August 18, 2005
 * 
 */

using System;
using System.Globalization;
using System.Resources;

namespace Cavingdeep.Util {
   /// <summary>
   /// This class reads resource strings.
   /// </summary>
   internal sealed class MessageReader {
      private const string DEFAULT_RESOURCE_BUNDLE = "Messages";
      private static ResourceManager resourceManager = new ResourceManager(DEFAULT_RESOURCE_BUNDLE, typeof(MessageReader).Assembly);

      private MessageReader() {}

      /// <summary>
      /// Gets a resource string.
      /// </summary>
      /// <param name="name">Name used to retrieve the resource.</param>
      /// <returns>Resource string retrieved.</returns>
      internal static string GetString(string name) {
         return MessageReader.resourceManager.GetString(name);
      }

      /// <summary>
      /// Gets a formatted resource string.
      /// </summary>
      /// <param name="name">Name used to retrieve the resource.</param>
      /// <param name="args">Arguments used for string formatting.</param>
      /// <returns>Formatted resource string retrieved.</returns>
      internal static string GetFormattedString(string name, params object[] args) {
         string message = MessageReader.resourceManager.GetString(name);

         return string.Format(CultureInfo.InvariantCulture, message, args);
      }
   }
}
