/*
 * Copyright (C) 2005, Wei Yuan.
 * All rights reserved.
 * 
 * Author: Wei Yuan
 * Created on: August 18, 2005
 * 
 */

using System;
using System.Data;

using NUnit.Framework;

using Rhino.Mocks;

using Cavingdeep.DbTestSample;

namespace Cavingdeep.Tests.DbTestSample {
   [TestFixture]
   public class EmployeeInfoTester {
      [Test]
      public void GetEmployeeInfo() {
         using (MockRepository mocks = new MockRepository()) {
            IDbConnection connection = (IDbConnection) mocks.CreateMock(typeof(IDbConnection));
            IDbCommand queryEmployeeCommand = (IDbCommand) mocks.CreateMock(typeof(IDbCommand));
            IDataReader reader = (IDataReader) mocks.CreateMock(typeof(IDataReader));

            connection.Open();
            connection.Close();

            SetupResult.On(queryEmployeeCommand).Call(queryEmployeeCommand.Connection).Return(connection);
            SetupResult.On(queryEmployeeCommand).Call(queryEmployeeCommand.ExecuteReader()).Return(reader);
            queryEmployeeCommand.CommandText = null;
            LastCall.On(queryEmployeeCommand).Constraints(Is.TypeOf(typeof(string)));

            SetupResult.On(reader).Call(reader.Read()).Return(true);
            SetupResult.On(reader).Call(reader.GetString(0)).Return("2005-08-18-001");
            SetupResult.On(reader).Call(reader.GetString(1)).Return("Juan");
            SetupResult.On(reader).Call(reader.GetString(2)).Return("Sola");
            SetupResult.On(reader).Call(reader.GetDateTime(3)).Return(new DateTime(1978, 10, 12));

            mocks.ReplayAll();

            EmployeeInfo employeeInfo = EmployeeInfo.GetEmployeeInfo("2005-08-18-001",
                                                                     queryEmployeeCommand);

            Assert.IsNotNull(employeeInfo);
            Assert.AreEqual("Juan", employeeInfo.FirstName);
            Assert.AreEqual("Sola", employeeInfo.LastName);
            Assert.AreEqual(new DateTime(1978, 10, 12), employeeInfo.Birthday);
         }
      }

      [Test]
      [ExpectedException(typeof(NoConnectionException))]
      public void GetEmployeeInfoWhenNoConnectionOpened() {
         using (MockRepository mocks = new MockRepository()) {
            IDbConnection connection = (IDbConnection) mocks.CreateMock(typeof(IDbConnection));
            IDbCommand queryCommand = (IDbCommand) mocks.CreateMock(typeof(IDbCommand));

            connection.Open();
            LastCall.On(connection).Throw(new NoConnectionException());

            SetupResult.On(queryCommand).Call(queryCommand.Connection).Return(connection);
            queryCommand.CommandText = null;
            LastCall.On(queryCommand).Constraints(Is.TypeOf(typeof(string)));

            mocks.ReplayAll();

            EmployeeInfo employeeInfo = EmployeeInfo.GetEmployeeInfo("2005-08-09-009", queryCommand);
         }
      }

      [Test]
      public void GetEmployeeInfoWithNoMatch() {
         using (MockRepository mocks = new MockRepository()) {
            IDbConnection connection = (IDbConnection) mocks.CreateMock(typeof(IDbConnection));
            IDbCommand queryEmployeeCommand = (IDbCommand) mocks.CreateMock(typeof(IDbCommand));
            IDataReader reader = (IDataReader) mocks.CreateMock(typeof(IDataReader));

            connection.Open();
            connection.Close();

            SetupResult.On(queryEmployeeCommand).Call(queryEmployeeCommand.Connection).Return(connection);
            SetupResult.On(queryEmployeeCommand).Call(queryEmployeeCommand.ExecuteReader()).Return(reader);
            queryEmployeeCommand.CommandText = null;
            LastCall.On(queryEmployeeCommand).Constraints(Is.TypeOf(typeof(string)));

            SetupResult.On(reader).Call(reader.Read()).Return(false);

            mocks.ReplayAll();

            EmployeeInfo employeeInfo = EmployeeInfo.GetEmployeeInfo("2005-09-21-046", queryEmployeeCommand);

            Assert.IsNull(employeeInfo);
         }
      }

      [Test]
      [ExpectedException(typeof(Exception))]
      public void GetEmployeeInfoWithWrongDatabase() {
         using (MockRepository mocks = new MockRepository()) {
            IDbConnection connection = (IDbConnection) mocks.CreateMock(typeof(IDbConnection));
            IDbCommand queryEmployeeCommand = (IDbCommand) mocks.CreateMock(typeof(IDbCommand));

            connection.Open();
            connection.Close();

            SetupResult.On(queryEmployeeCommand).Call(queryEmployeeCommand.Connection).Return(connection);
            SetupResult.On(queryEmployeeCommand).Call(queryEmployeeCommand.ExecuteReader())
                       .Throw(new Exception());
            queryEmployeeCommand.CommandText = null;
            LastCall.On(queryEmployeeCommand).Constraints(Is.TypeOf(typeof(string)));

            mocks.ReplayAll();

            EmployeeInfo employeeInfo = EmployeeInfo.GetEmployeeInfo("2005-09-21-046", queryEmployeeCommand);
         }
      }
   }
}